@@ -1,25 +1,34 @@
 from enum import Enum
 import ipywidgets
-from functools import partial 
-from ipywidgets import HBox, VBox ,Layout, Label
-PlayDirection = Enum('PlayDirection','Forward Backward None')
+from types import MethodType
+from functools import partial
+from ipywidgets import HBox, VBox, Layout, Label
+
+PlayDirection = Enum("PlayDirection", "Forward Backward None")
 from threading import *
 from .timer import PerpetualTimer
 from traitlets import *
 import locale
+from delpy import DelpyWidget
+import functools
 
 global translation
-translation = {'fr_FR':{'time':'Temps :', 'speed':'Vitesse :'},'en_EN':{'time':'Time :', 'speed':'Speed :'}}
+translation = {
+    "fr_FR": {"time": "Temps :", "speed": "Vitesse :"},
+    "en_EN": {"time": "Time :", "speed": "Speed :"},
+}
 
-__version__ = '0.1'
+__version__ = "0.1"
 
-class Player(HasTraits) :
+
+class Player(HasTraits):
     """
         A Player keep an history of the _view values so the user may check the steps leading to a final result
     """
-    time=Int()
-    action=traitlets.Unicode()
-        
+
+    time = Int()
+    action = traitlets.Unicode()
+
     def __init__(self, _view, language=None):
         """
             The Player's initialization
@@ -51,42 +60,41 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        ## If no language is specified, we set it to the system's language
-        self.language='en_EN'
-        if not(language in translation):
+        self.language = "en_EN"
+        if not (language in translation):
             if locale.getdefaultlocale()[0] in translation:
-                self.language=locale.getdefaultlocale()[0]
+                self.language = locale.getdefaultlocale()[0]
         else:
-            self.language=language
-            
+            self.language = language
+
         self.view = _view
-        self.original_value=self.view.value
-        self.play_direction=PlayDirection.Forward
-        self.play_fps=1
-        
-        self.timer =PerpetualTimer( self.play_fps,self.tick )
-        self.history=[]
-        self.actionHistory=[]
-        self.time=0
-        self.playing=True
-                   
-        style = {'description_width': 'initial'}    
-        self.slider_time=ipywidgets.IntSlider(
+        self.original_value = self.view.value
+        self.play_direction = PlayDirection.Forward
+        self.play_fps = 1
+
+        self.timer = PerpetualTimer(self.play_fps, self.tick)
+        self.history = []
+        self.actionHistory = []
+        self.time = 0
+        self.playing = True
+
+        style = {"description_width": "initial"}
+        self.slider_time = ipywidgets.IntSlider(
             value=self.time,
             min=0,
-            max=0,#len(self.history),
+            max=0, 
             step=1,
-            description=translation[self.language]['time'],
-            style=style
+            description=translation[self.language]["time"],
+            style=style,
         )
         self.reset(self.original_value)
-            
-        # Slider to control which step to display
+
         def on_value_change(change):
-            self.time=change['new']
+            self.time = change["new"]
             self.update()
-        self.slider_time.observe(on_value_change,names='value')
-    
+
+        self.slider_time.observe(on_value_change, names="value")
+
     def run(self):
         """
             Run the timer
@@ -106,7 +114,7 @@ class Player(HasTraits) :
             
         """
         self.timer.run()
-    
+
     def update(self):
         """
             Update the Player's value and action according to self.time
@@ -127,11 +135,11 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        self.slider_time.value=self.time
-        self.view.value= self.history[self.time]
-        if len(self.actionHistory)>self.time:
-            self.action=self.actionHistory[self.time]
-    
+        self.slider_time.value = self.time
+        self.view.value = self.history[self.time]
+        if len(self.actionHistory) > self.time:
+            self.action = self.actionHistory[self.time]
+
     def get_value(self):
         """
             Get the last value in the history
@@ -146,9 +154,9 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        return self.history[len(self.history)-1]
-    
-    def set_action(self,newAction):
+        return self.history[len(self.history) - 1]
+
+    def set_action(self, newAction):
         """
             Add a new action to the history
             
@@ -165,12 +173,12 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        if (len(self.actionHistory) > 10000):
+        if len(self.actionHistory) > 10000:
             raise RuntimeError("Votre programme a pris plus de 1000 étapes")
-        self.action=newAction
+        self.action = newAction
         self.actionHistory.append(newAction)
-    
-    def set_value(self,value): 
+
+    def set_value(self, value):
         """
             Add a new value to the history
             
@@ -190,15 +198,14 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        if (len(self.history) > 10000):
+        if len(self.history) > 10000:
             raise RuntimeError("Votre programme a pris plus de 1000 étapes")
         self.history.append(value)
-        self.slider_time.max=len(self.history)-1
-        if ( not self.timer.running() and self.time == len(self.history) -2):
-            self.time=self.time+1
+        self.slider_time.max = len(self.history) - 1
+        if not self.timer.running() and self.time == len(self.history) - 2:
+            self.time = self.time + 1
             self.update()
-    
-    
+
     def tick(self):
         """
             Make the timer either increase or decrease
@@ -217,12 +224,12 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        if(self.play_direction == PlayDirection.Forward ):  
+        if self.play_direction == PlayDirection.Forward:
             self.step_forward()
         else:
             self.step_backward()
-    
-    def reset(self,value):    
+
+    def reset(self, value):
         """
             Reset the Player and set its first value to 'value'
             
@@ -251,12 +258,12 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        self.time=0
-        self.history=[value]
-        self.actionHistory=['']
-        self.slider_time.max=len(self.history)-1
+        self.time = 0
+        self.history = [value]
+        self.actionHistory = [""]
+        self.slider_time.max = len(self.history) - 1
         self.update()
-    
+
     def begin(self):
         """
             Set the Player to the first step
@@ -283,9 +290,9 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        self.time=0
+        self.time = 0
         self.update()
-    
+
     def end(self):
         """
             Set the Player to the last step
@@ -309,7 +316,7 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        self.time= len(self.history)-1  
+        self.time = len(self.history) - 1
         self.update()
 
     def step_backward(self):
@@ -337,10 +344,10 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        if( self.time >0):
-            self.time=self.time-1
+        if self.time > 0:
+            self.time = self.time - 1
             self.update()
-            
+
     def step_forward(self):
         """
             Set the Player to the next step
@@ -366,45 +373,42 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        if( self.time <len(self.history)-1):  
-            self.time=self.time+1
-            self.update()        
-            
-    def backward (self):
+        if self.time < len(self.history) - 1:
+            self.time = self.time + 1
+            self.update()
+
+    def backward(self):
         """
             Timer's steps will decrease
         """
-        self.play_direction = PlayDirection.Backward # a check
+        self.play_direction = PlayDirection.Backward  
         self.timer.set_fps(self.play_fps)
         self.play()
-    
+
     def forward(self):
         """
             Timer's step will increase
         """
-        self.play_direction = PlayDirection.Forward # a check
+        self.play_direction = PlayDirection.Forward 
         self.timer.set_fps(self.play_fps)
         self.play()
-    
+
     def play(self):
         """
             Start the Player's timer
         """
-        #self. play_direction = PlayDirection.Forward # a check
         self.timer.set_fps(self.play_fps)
-        self.playing=True
+        self.playing = True
         self.timer.run()
 
     def pause(self):
         """
             Stop the Player's timer
         """
-        #self.play_direction=None
         self.timer.cancel()
-        self.playing=False
-    
-    
-    def set_fps(self,fps):
+        self.playing = False
+
+    def set_fps(self, fps):
         """
             Set the FPS
             
@@ -413,14 +417,11 @@ class Player(HasTraits) :
             fps : (type=int) new FPS value
             
         """
-        self.play_fps=fps
-        if(self.play_direction != None):
+        self.play_fps = fps
+        if self.play_direction != None:
             self.timer.set_fps(fps)
-    
-    #def won(self):
-    #    return self.history[len(self.history)-1].won()
 
-    def set_time(self,tps): 
+    def set_time(self, tps):
         """
             Set the Player's time
             
@@ -445,141 +446,151 @@ class Player(HasTraits) :
             >>> player.pause()
             
         """
-        self.time=tps
+        self.time = tps
 
 
-class PlayerView(VBox):
+#class PlayerView(VBox):
+class ValuePlayerWidget(VBox):
     """
         The Player's visualisation
     """
-    def __init__(self,player,buttons=[]):
+    def __init__(self,visualisation, UI=None, actions=None, language=None):
         """
             Initialization of PlayerView
             
             Parameters
             ----------
-            player : (type=Player) the Player we make the visualisation for
-            buttons : (default value=[]) (type=list) the list of action buttons we have to make
+            visualisation : (type=traitlets) the visualisation we link the player with
+            UI : (default value=None) (type=string) The type of User Interface
+            actions : (type=dictionnary) a dictionnary of lists. Each list contains tuples. Each tupple defines an action and fit the following format : ('function_name', fun, *args) 
+            language : (type=None) language for displayed labels
             
         """
-        self.widget=ipywidgets.HTML(
-        value="",
-        placeholder='',
-        description='',
-        )
+        self.widget = ipywidgets.HTML(value="", placeholder="", description="",)
         output = ipywidgets.Output()
-        self.player=player
-        
-        # on_clicked functions to control the timer
+        self.player = Player(visualisation, language)
+
         def fast_backward_clicked(b):
             with output:
                 self.player.begin()
                 self.player.pause()
                 self.color(b)
-                self.pause.icon='play'
-            
+                self.pause.icon = "play"
+
         def backward_clicked(b):
             with output:
-                self.slider.value=0
+                self.slider.value = 0
                 self.color(b)
-                self.pause.icon='pause'
+                self.pause.icon = "pause"
                 self.player.backward()
-                
+
         def step_backward_clicked(b):
-            with output: 
+            with output:
                 self.player.pause()
                 self.player.step_backward()
                 self.color(b)
-                self.pause.icon='play'
+                self.pause.icon = "play"
 
         def step_forward_clicked(b):
             with output:
                 self.player.pause()
                 self.player.step_forward()
                 self.color(b)
-                self.pause.icon='play'
-
+                self.pause.icon = "play"
 
         def pause_clicked(b):
             with output:
                 if self.player.playing:
                     self.player.pause()
                     self.color(b)
-                    b.icon='play'
+                    b.icon = "play"
                 else:
                     self.player.play()
                     self.color(b)
-                    b.icon='pause'
-                    
-        
+                    b.icon = "pause"
+
         def fast_forward_clicked(b):
             with output:
                 self.player.pause()
                 self.player.end()
                 self.color(b)
-                self.pause.icon='play'
-                
+                self.pause.icon = "play"
+
         def forward_clicked(b):
             with output:
-                self.slider.value=0
+                self.slider.value = 0
                 self.color(b)
-                self.pause.icon='pause'
+                self.pause.icon = "pause"
                 self.player.forward()
-        
+
         def speed_forward_clicked(b):
             with output:
-                if self.player.play_direction==PlayDirection.Forward:
-                    self.slider.value+=1
+                if self.player.play_direction == PlayDirection.Forward:
+                    self.slider.value += 1
                 else:
                     self.player.forward()
-                    self.slider.value=1
+                    self.slider.value = 1
                 self.color(b)
-                self.pause.icon='pause'
-        
+                self.pause.icon = "pause"
+
         def speed_backward_clicked(b):
             with output:
-                if self.player.play_direction==PlayDirection.Forward:
+                if self.player.play_direction == PlayDirection.Forward:
                     self.player.backward()
-                    self.slider.value=1
+                    self.slider.value = 1
                 else:
-                    self.slider.value+=1
+                    self.slider.value += 1
                 self.color(b)
-                self.pause.icon='pause'
-                
-                
-        
-        ## Speed slider
-        style = {'description_width': 'initial'}
-        self.slider=ipywidgets.FloatSlider(
-        value=1.0,
-        min=0.0,
-        max=6.0,
-        step=1,
-            description=translation[self.player.language]['speed'],
-        style=style
+                self.pause.icon = "pause"
+
+        style = {"description_width": "initial"}
+        self.slider = ipywidgets.FloatSlider(
+            value=1.0,
+            min=0.0,
+            max=6.0,
+            step=1,
+            description=translation[self.player.language]["speed"],
+            style=style,
         )
-        # when the slider's value changes, the fps of the timer change 
         def on_value_change(change):
-            value=int (change['new'])
-            self.player.set_fps(2**value)
-        
-        self.slider.observe(on_value_change, names='value')
-        
+            value = int(change["new"])
+            self.player.set_fps(2 ** value)
+
+        self.slider.observe(on_value_change, names="value")
+
         # Buttons for the timer
-        self.speed_forward= ipywidgets.Button(description="",icon='forward',layout=Layout(width='35px'))  
-        self.speed_backward= ipywidgets.Button(description="",icon='backward',layout=Layout(width='35px')) 
-        
-        self.forward= ipywidgets.Button(description="",icon='chevron-right',layout=Layout(width='35px')) 
-        self.backward= ipywidgets.Button(description="",icon='chevron-left',layout=Layout(width='35px'))
-        
-        self.pause= ipywidgets.Button(description="",icon='pause',layout=Layout(width='35px'))
-        
-        self.fast_backward= ipywidgets.Button(description="",icon='fast-backward',layout=Layout(width='35px'))
-        self.fast_forward= ipywidgets.Button(description="",icon='fast-forward',layout=Layout(width='35px'))
-        
-        self.step_backward= ipywidgets.Button(description="",icon='step-backward',layout=Layout(width='35px'))
-        self.step_forward= ipywidgets.Button(description="",icon='step-forward',layout=Layout(width='35px'))
-        
+        self.speed_forward = ipywidgets.Button(
+            description="", icon="forward", layout=Layout(width="35px")
+        )
+        self.speed_backward = ipywidgets.Button(
+            description="", icon="backward", layout=Layout(width="35px")
+        )
+
+        self.forward = ipywidgets.Button(
+            description="", icon="chevron-right", layout=Layout(width="35px")
+        )
+        self.backward = ipywidgets.Button(
+            description="", icon="chevron-left", layout=Layout(width="35px")
+        )
+
+        self.pause = ipywidgets.Button(
+            description="", icon="pause", layout=Layout(width="35px")
+        )
+
+        self.fast_backward = ipywidgets.Button(
+            description="", icon="fast-backward", layout=Layout(width="35px")
+        )
+        self.fast_forward = ipywidgets.Button(
+            description="", icon="fast-forward", layout=Layout(width="35px")
+        )
+
+        self.step_backward = ipywidgets.Button(
+            description="", icon="step-backward", layout=Layout(width="35px")
+        )
+        self.step_forward = ipywidgets.Button(
+            description="", icon="step-forward", layout=Layout(width="35px")
+        )
+
         # on_click function for the timer's buttons
         self.pause.on_click(pause_clicked)
         self.fast_backward.on_click(fast_backward_clicked)
@@ -590,49 +601,96 @@ class PlayerView(VBox):
         self.fast_forward.on_click(fast_forward_clicked)
         self.speed_forward.on_click(speed_forward_clicked)
         self.speed_backward.on_click(speed_backward_clicked)
-        
-        
-        self.buttonList=[self.pause, self.fast_backward, self.backward, self.forward, self.step_backward, self.step_forward, self.fast_forward, self.speed_forward, self.speed_backward]
-        
+
+        self.buttonList = [
+            self.pause,
+            self.fast_backward,
+            self.backward,
+            self.forward,
+            self.step_backward,
+            self.step_forward,
+            self.fast_forward,
+            self.speed_forward,
+            self.speed_backward,
+        ]
+
         for b in self.buttonList:
-            b.style.button_color='WhiteSmoke'
-       
-        
-        
+            b.style.button_color = "WhiteSmoke"
+
         # action label
-        actionHead=ipywidgets.Label(value=" Action : ")
-        actionLabel=ipywidgets.Label(value=self.player.action)
-        actionBox=HBox([actionHead,actionLabel])#,layout=ipywidgets.Layout(justify_content="center"))
-        
+        actionHead = ipywidgets.Label(value=" Action : ")
+        actionLabel = ipywidgets.Label(value=self.player.action)
+        actionBox = HBox(
+            [actionHead, actionLabel]
+        ) 
+
         # When the player's action changes, the display is updated
         def on_action_change(change):
-            actionLabel.value=change['new']
-        
-        traitlets.link([actionLabel, "value"], [self.player,"action"])
-        
+            actionLabel.value = change["new"]
+
+        traitlets.link([actionLabel, "value"], [self.player, "action"])
+        self.affichage = ipywidgets.HBox(
+            [
+                self.fast_backward,
+                self.speed_backward,
+                self.backward,
+                self.step_backward,
+                self.pause,
+                self.step_forward,
+                self.forward,
+                self.speed_forward,
+                self.fast_forward,
+                actionBox,
+            ]
+        )
+
+        upBox = HBox([self.player.slider_time, self.slider])
         # Action Buttons
-        buttonTab=[]
-        for tab in buttons :
-            mytab=[]
-            for act in tab:
-                mytab.append(self.mk_button(*act))
-            buttonTab.append(VBox(mytab))
-            
-        buttonBox=HBox(buttonTab)
-        # Initialization as a VBox
-        self.affichage=ipywidgets.HBox([self.fast_backward,self.speed_backward,self.backward,self.step_backward,self.pause,self.step_forward,self.forward,self.speed_forward,self.fast_forward,actionBox])
-        upBox=HBox([self.player.slider_time,self.slider])
-        VBox.__init__(self,[self.player.view,upBox,self.affichage,buttonBox])
-    
-    def color(self,myButton):
+
+        self.delpy = None
+        if UI == "btn":
+            buttonTab = []
+            for cat in actions:
+                mytab = []
+                for act in actions[cat]:
+                    mytab.append(self.mk_button(*act))
+                buttonTab.append(VBox(mytab))
+            buttonBox = HBox(buttonTab)
+            VBox.__init__(self, [self.player.view, upBox, self.affichage, buttonBox])
+        elif UI == "delpy" and actions != None:
+            self.delpy = DelpyWidget()
+            setattr(self.delpy, "player", self)
+            for tag in actions:
+                for i in actions[tag]:
+                    k = i[0].replace(" ", "_").lower()
+                    v = i[1]
+                    if len(i) > 2:
+                        arguments = i[2:]
+                        v = functools.partial(v, *arguments)
+                    else:
+                        v = functools.partial(v)
+                    setattr(v, "_delpy", tag)
+                    sig = inspect.signature(v)
+                    parameters = list(sig.parameters.keys())
+                    self.delpy._procedures[k] = {
+                        "args": [],
+                        "ret": None,
+                        "doc": v.__doc__,
+                        "category": str(getattr(v, "_delpy")),
+                    }
+                    setattr(self.delpy, k, v)
+            VBox.__init__(self, [self.player.view, upBox, self.affichage, self.delpy])
+        else:
+            VBox.__init__(self, [self.player.view, upBox, self.affichage])
+
+    def color(self, myButton):
         for b in self.buttonList:
             if b.icon != myButton.icon:
-                b.style.button_color='WhiteSmoke'
+                b.style.button_color = "WhiteSmoke"
             else:
-                b.style.button_color='#D3D3D3'
-        #myButton.style.button_color='#D3D3D3'
+                b.style.button_color = "#D3D3D3"
 
-    def set_value(self,value):
+    def set_value(self, value):
         """
             Set the value of the Player
             
@@ -642,37 +700,22 @@ class PlayerView(VBox):
             
         """
         self.player.set_value(value)
-    
+
     def pause(self):
         """
             Stop the player
         """
         self.player.pause()
-    
+
     def mk_button(self, action, func, *args):
         """ Make an action button """
-        name=action
+        name = action
         button = ipywidgets.Button(description=name)
         output = ipywidgets.Output()
+
         def on_button_clicked(b):
             with output:
-                self.set_value(func(*args))
-                self.player.set_action(name)
-        button.on_click(on_button_clicked)
-        return button
+                func(*args)
 
-    
-def ValuePlayerWidget(visualisation, actions=[], language=None):
-    """
-        Returns a PlayerView 
-        
-        Parameters
-        -----------
-        visualisation : (type=traitlets) the visualisation we link the player with
-        buttons : (type=list) a list of lists. Each inside list is meant to be a VBox of buttons. Each element of the list has to be under the format (name, function) or (name, function, value). 'name' specifies the name of the action, 'function' is the function the buttons must execute when clicked (function must return a value for player.value), 'value' is not obligatory, it is the given value to 'function'
-        language : (type=None) language for displayed labels
-        
-    """
-    player=Player(visualisation,language)
-    app=PlayerView(player, actions) 
-    return app
+        button.on_click(on_button_clicked)
+        return button
\ No newline at end of file
